using System;
using Jypeli;
using Jypeli.Widgets;

namespace Swatteri;

/// @author rtmat
/// @version 4.3.2025
/// <summary>
///  Peli, jossa tarkoitus tuhota mahdollisimman monta ampparia minuutissa.
///  Amppareita saa tuhottua viemällä hiiri sellaisen kohdalle
/// </summary>
public class Swatteri : PhysicsGame
{
    private readonly Image _latkanKuva = LoadImage("lätkä.png");
    private readonly Image _amppariKuva = LoadImage("amppari.png");
    private readonly Image _taustaKuva = LoadImage("Taustakuva.png");
    private readonly Image _ampparihit = LoadImage("ampparihit.png");
    private PhysicsObject _latka;
    private DoubleMeter _laskuri;
    private Timer _aikalaskuri;
    private EasyHighScore _topLista = new EasyHighScore();
    private IntMeter _pistelaskuri;
    private Vector[] _tuhottujenSijainti = new Vector[200];
    private int _tuhottuja;

    
    /// <summary>
    /// Luodaan peli
    /// </summary>
    public override void Begin()
    {

        Mouse.ListenMovement(0.1, Liikuta, "Liikuta lätkää");
        Mouse.IsCursorVisible = false;
        Kentta();
        LuoLaskuri();
        _latka = LuoSwatter(this, 0.0, 0.0);
        LuoAmpparit();
        LuoPistelaskuri();

        foreach (Vector position in _tuhottujenSijainti)
        {
            if (position != Vector.Zero)
            {
                PhysicsObject kuollut = new PhysicsObject(30.0, 30.0);
                kuollut.Position = position;
                kuollut.Shape = Shape.Circle;
                kuollut.Color = Color.LightYellow;
                kuollut.IgnoresCollisionResponse = true;
                kuollut.LifetimeLeft = TimeSpan.FromSeconds(1);
                kuollut.Image = _ampparihit;
                Add(kuollut);
            }
        }

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }
    

    /// <summary>
    /// Luodaan kenttä, jossa peliä pelataan
    /// </summary>
    private void Kentta()
    {
        Level.Background.Image = _taustaKuva;

        Double kitka = 0.0;
        Double kimmoke = 1.0;

        PhysicsObject vasen = Level.CreateLeftBorder();
        vasen.Restitution = kimmoke;
        vasen.KineticFriction = kitka;
        vasen.IsVisible = false;

        PhysicsObject oikea = Level.CreateRightBorder();
        oikea.Restitution = kimmoke;
        oikea.KineticFriction = kitka;
        oikea.IsVisible = false;

        PhysicsObject yla = Level.CreateTopBorder();
        yla.Restitution = kimmoke;
        yla.KineticFriction = kitka;
        yla.IsVisible = false;

        PhysicsObject ala = Level.CreateBottomBorder();
        ala.Restitution = kimmoke;
        ala.KineticFriction = kitka;
        ala.IsVisible = false;

        Camera.ZoomToLevel();
    }
    
    
    /// <summary>
    /// Luodaan pelille peliaika
    /// </summary>
    void LuoLaskuri()
    {
        _laskuri = new DoubleMeter(60.0);
        
        _aikalaskuri = new Timer();
        _aikalaskuri.Interval = 0.1;
        _aikalaskuri.Timeout += LaskeAlaspain;
        _aikalaskuri.Start();

        Label aikanaytto = new Label();
        aikanaytto.TextColor = Color.Red;
        aikanaytto.Color = Color.Black;
        aikanaytto.DecimalPlaces = 1;
        aikanaytto.BindTo(_laskuri);
        aikanaytto.X = 400;
        aikanaytto.Y = 340;
        Add(aikanaytto);
    }
    
    
    /// <summary>
    /// Alaspäin laskeva aika, peliaika aliohjelmalle
    /// </summary>
    void LaskeAlaspain()
    {
        _laskuri.Value -= 0.1;
        
        if (_laskuri.Value <= 0)
        {
            MessageDisplay.Add("Peli päättyi");
            _aikalaskuri.Stop();
            _topLista.EnterAndShow(_pistelaskuri.Value);
            _topLista.HighScoreWindow.Closed += AloitaPeli;
            
        }
        
    }
    
    
    /// <summary>
    /// Luodaan pistelaskuri, joka lastee tuhottujen amppareiden määrän
    /// </summary>
    void LuoPistelaskuri()
    {
        _pistelaskuri = new IntMeter(0);               
      
        Label pistenaytto = new Label(); 
        pistenaytto.X = Screen.Left + 60;
        pistenaytto.Y = Screen.Top - 40;
        pistenaytto.TextColor = Color.Black;
        pistenaytto.Color = Color.White;

        pistenaytto.BindTo(_pistelaskuri);
        Add(pistenaytto);
    }
    
    
    /// <summary>
    /// Pelin aloitus
    /// </summary>
    /// <param name="sender">Uusi peli-ikkuna</param>
    public void AloitaPeli(Window sender)
    {
        ClearAll();
        Begin();
    }

    
    /// <summary>
    /// Luodaan lätkä, jolla amppareita lyödään
    /// </summary>
    /// <param name="x">lätkän y-koordinaatti</param>
    /// <param name="y">lätkän x-koordinaatti</param>
    /// <param name="peli">peli, jonne lätkä</param>
    public PhysicsObject LuoSwatter(PhysicsGame peli, double x, double y)
    {
        PhysicsObject swatter = new PhysicsObject(90.0, 90.0, Shape.Circle);
        swatter.X = x;
        swatter.Y = y;
        swatter.Image = _latkanKuva;
        swatter.IgnoresCollisionResponse = true;

        peli.Add(swatter);
        return swatter;
    }

    
    /// <summary>
    /// Luodaan amppari, jota yritetään lyödä lätkällä
    /// </summary>
    /// <param name="x">ei käytössä</param>
    /// <param name="y">ei käytössä</param>
    /// <param name="peli">peli, jonne amppari</param>

    public PhysicsObject LuoAmppari(PhysicsGame peli, double x, double y)
    {
        PhysicsObject bee = new PhysicsObject(30.0, 30.0, Shape.Circle);
        bee.X = RandomGen.NextDouble(-300, 300);
        bee.Y = RandomGen.NextDouble(-300, 300);
        bee.Image = _amppariKuva;
        
        RandomMoverBrain ampparinaivot = new RandomMoverBrain(300); // satunnainen liike
        bee.Brain = ampparinaivot;
        bee.CollisionIgnoreGroup = 1;
        
        peli.Add(bee);
        AddCollisionHandler(_latka, bee, Tuhoa);
        return bee;
    }
    

    /// <summary>
    /// Liike liikutettavalle lätkälle
    /// </summary>
    public void Liikuta()
    {
        _latka.Position = Mouse.PositionOnWorld;
    }
    

    /// <summary>
    /// Ampiaisen tuhoaminen lätkällä
    /// </summary>
    /// <param name="lätkä">ei käytössä</param>
    /// <param name="kohde">kohde, joka tuhotaan (ampiainen)</param>
    public void Tuhoa(PhysicsObject lätkä, PhysicsObject kohde)
    {
        if (_tuhottuja < _tuhottujenSijainti.Length)
        {
            _tuhottujenSijainti[_tuhottuja] = kohde.Position;
            _tuhottuja++;
        }
        
        kohde.Destroy();
        _pistelaskuri.Value += 1;
        if (_pistelaskuri.Value % 6 == 0)
        {
            LuoAmpparit();
        }
    }
    
    
    /// <summary>
    /// Luodaan ampparit peliin.
    /// </summary>
    public void LuoAmpparit()
    {
        for (int i = 0; i <= 5; i++)
        {
            LuoAmppari(this, RandomGen.NextDouble(-300, 300), RandomGen.NextDouble(-300, 300));
        }
    }
    
    
}    